@@ -12,8 +12,10 @@
 # License for the specific language governing permissions and limitations under
 # the License.
 
+import abc
 import parameter
 import warnings
+import traceback
 
 Parameter = parameter.Parameter
 
@@ -22,26 +24,29 @@ def namespace(namespace=None):
     """ Call to set namespace of tasks declared after the call.
 
     If called without arguments or with None as the namespace, the namespace is reset, which is recommended to do at the end of any file where the namespace is set to avoid unintentionally setting namespace on tasks outside of the scope of the current file."""
-    TaskMetaclass._default_namespace = namespace
+    Register._default_namespace = namespace
 
 
-class TaskMetaclass(type):
-    # If we already have an instance of this class, then just return it from the cache
-    # The idea is that a Task object X should be able to set up heavy data structures that
-    # can be accessed from other Task objects (with dependencies on X). But we need to make
-    # sure that X is not instantiated many times.
+class Register(abc.ABCMeta):
+    # 1. Cache instances of objects so that eg. X(1, 2, 3) always returns the same object
+    # 2. Keep track of all subclasses of Task and expose them
     __instance_cache = {}
     _default_namespace = None
+    _reg = []
+    AMBIGUOUS_CLASS = object()  # Placeholder denoting an error
 
     def __new__(metacls, classname, bases, classdict):
-        """ Custom class creation for namespacing
-
-        Set the task namespace to whatever the currently declared namespace is"""
+        """ Custom class creation for namespacing. Also register all subclasses
 
+        Set the task namespace to whatever the currently declared namespace is
+        """
         if "task_namespace" not in classdict:
             classdict["task_namespace"] = metacls._default_namespace
 
-        return type.__new__(metacls, classname, bases, classdict)
+        cls = super(Register, metacls).__new__(metacls, classname, bases, classdict)
+        if cls.run != NotImplemented:
+            metacls._reg.append(cls)
+        return cls
 
     def __call__(cls, *args, **kwargs):
         """ Custom class instantiation utilizing instance cache.
@@ -49,9 +54,9 @@ class TaskMetaclass(type):
         If a Task has already been instantiated with the same parameters,
         the previous instance is returned to reduce number of object instances."""
         def instantiate():
-            return super(TaskMetaclass, cls).__call__(*args, **kwargs)
+            return super(Register, cls).__call__(*args, **kwargs)
 
-        h = TaskMetaclass.__instance_cache
+        h = Register.__instance_cache
 
         if h == None:  # disabled
             return instantiate()
@@ -61,6 +66,11 @@ class TaskMetaclass(type):
 
         k = (cls, tuple(param_values))
 
+        try:
+            hash(k)
+        except TypeError:
+            return instantiate()  # unhashable types in parameters
+
         if k not in h:
             h[k] = instantiate()
 
@@ -68,11 +78,11 @@ class TaskMetaclass(type):
 
     @classmethod
     def clear_instance_cache(self):
-        TaskMetaclass.__instance_cache = {}
+        Register.__instance_cache = {}
 
     @classmethod
     def disable_instance_cache(self):
-        TaskMetaclass.__instance_cache = None
+        Register.__instance_cache = None
 
     @property
     def task_family(cls):
@@ -81,9 +91,33 @@ class TaskMetaclass(type):
         else:
             return "%s.%s" % (cls.task_namespace, cls.__name__)
 
+    @classmethod
+    def get_reg(cls):
+        reg = {}
+        for cls in cls._reg:
+            name = cls.task_family
+            if name in reg and reg[name] != cls:
+                # Registering two different classes - this means we can't instantiate them by name
+                reg[name] = cls.AMBIGUOUS_CLASS
+            else:
+                reg[name] = cls
+
+        return reg
+
+    @classmethod
+    def get_global_params(cls):
+        global_params = {}
+        for cls in cls._reg:
+            for param_name, param_obj in cls.get_global_params():
+                if param_name in global_params and global_params[param_name] != param_obj:
+                    # Could be registered multiple times in case there's subclasses
+                    raise Exception('Global parameter %r registered by multiple classes' % param_name)
+                global_params[param_name] = param_obj
+        return global_params.iteritems()
+
 
 class Task(object):
-    __metaclass__ = TaskMetaclass
+    __metaclass__ = Register
 
     """
     non-declared properties: (created in metaclass):
@@ -128,29 +162,33 @@ class Task(object):
 
         params_dict = dict(params)
 
+        # In case any exceptions are thrown, create a helpful description of how the Task was invoked
+        # TODO: should we detect non-reprable arguments? These will lead to mysterious errors
+        exc_desc = '%s[args=%s, kwargs=%s]' % (cls.__name__, args, kwargs)
+
         # Fill in the positional arguments
         positional_params = [(n, p) for n, p in params if not p.is_global]
         for i, arg in enumerate(args):
             if i >= len(positional_params):
-                raise parameter.UnknownParameterException('Class %s: takes at most %d parameters (%d given)' % (cls.__name__, len(positional_params), len(args)))
+                raise parameter.UnknownParameterException('%s: takes at most %d parameters (%d given)' % (exc_desc, len(positional_params), len(args)))
             param_name, param_obj = positional_params[i]
             result[param_name] = arg
 
         # Then the optional arguments
         for param_name, arg in kwargs.iteritems():
             if param_name in result:
-                raise parameter.DuplicateParameterException('Class %s: parameter %s was already set as a positional parameter' % (cls.__name__, param_name))
+                raise parameter.DuplicateParameterException('%s: parameter %s was already set as a positional parameter' % (exc_desc, param_name))
             if param_name not in params_dict:
-                raise parameter.UnknownParameterException('Class %s: unknown parameter %s' % (cls.__name__, param_name))
+                raise parameter.UnknownParameterException('%s: unknown parameter %s' % (exc_desc, param_name))
             if params_dict[param_name].is_global:
-                raise parameter.ParameterException('Class %s: can not override global parameter %s' % (cls.__name__, param_name))
+                raise parameter.ParameterException('%s: can not override global parameter %s' % (exc_desc, param_name))
             result[param_name] = arg
 
         # Then use the defaults for anything not filled in
         for param_name, param_obj in params:
             if param_name not in result:
                 if not param_obj.has_default:
-                    raise parameter.MissingParameterException("'%s' tasks requires the '%s' parameter to be set" % (cls.__name__, param_name))
+                    raise parameter.MissingParameterException("%s: requires the '%s' parameter to be set" % (exc_desc, param_name))
                 result[param_name] = param_obj.default
 
         def list_to_tuple(x):
@@ -178,6 +216,9 @@ class Task(object):
         self.task_id = '%s(%s)' % (self.task_family, ', '.join(task_id_parts))
         self.__hash = hash(self.task_id)
 
+    def initialized(self):
+        return hasattr(self, 'task_id')
+
     @classmethod
     def from_input(cls, params, global_params):
         # Creates an instance from a str->str hash (to be used for cmd line interaction etc)
@@ -231,15 +272,17 @@ class Task(object):
     def run(self):
         pass  # default impl
 
-    def on_failure(self, exception, traceback):
+    def on_failure(self, exception):
         """ Override for custom error handling
 
         This method gets called if an exception is raised in :py:meth:`run`.
-        Return value of this method is json encoded and sent to the scheduler as the `expl` argument.
-        Default behavior is to return a string representation of the exception and traceback.
+        Return value of this method is json encoded and sent to the scheduler as the `expl` argument. Its string representation will be used as the body of the error email sent out if any.
+
+        Default behavior is to return a string representation of the stack trace.
         """
-        return {"exception": str(exception),
-                "traceback": str(traceback)}
+
+        traceback_string = traceback.format_exc()
+        return "Runtime error:\n%s" % traceback_string
 
     def on_success(self):
         """ Override for doing custom completion handling for a larger class of tasks
